﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Globalization;
using System.Threading.Tasks;
using System.ComponentModel; //pro bindinglist
using System.Windows.Forms;
using Data;

namespace App
{
    /// <summary>
    /// Třída pro načtení uložených dat ze souboru
    /// </summary>
    public class DataLoader
    {
        /// <summary>
        /// Metoda načte uložené osoby (Person)
        /// <paramref name="fileType">string type</paramref>
        /// <param name="fileType">Typ souboru, který má být načten (txt, cvs, xml)</param>
        /// <paramref name="system">bool system</paramref>
        /// <param name="system">TRUE jedná-li se o načítání uložených dat při spuštění programu, FALSE otevře dialog</param>
        /// </summary>
        public BindingList<Person> loadPeopleFromFile(string fileType = "txt", bool system = true)
        {
            BindingList<Person> people = new BindingList<Person>();

            OpenFileDialog openFile = new OpenFileDialog();

            /* Nastavení typu souborů - zatím je možné importovat pouze programem vytvořené TXT soubory*/
            if(fileType.CompareTo("txt") == 0)
                openFile.Filter = "Text Files (*.txt)|*.txt";
            else if (fileType.CompareTo("cvs") == 0)
                openFile.Filter = "csv files (*.csv)|*.csv";
            else if (fileType.CompareTo("xml") == 0)
                openFile.Filter = "xml files (*.xml)|*.xml";
            else 
                openFile.Filter = "All Files (*.*)|*.*";

            System.Windows.Forms.DialogResult res;
            string filename;
            if (system)
            {
                res = DialogResult.OK;
                filename = "people.txt";
            }
            else
            {
                res = openFile.ShowDialog();
                filename = openFile.FileName;
            }
            
            if (res == System.Windows.Forms.DialogResult.OK)
            {
                try
                {
                    using (Stream file = new FileStream(filename, FileMode.Open, FileAccess.Read))
                    {
                        using (StreamReader reader = new StreamReader(file, Encoding.UTF8))
                        {
                            string line = "";
                            while ((line = reader.ReadLine()) != null)
                            {
                                String[] s = line.Split('|');
                                people.Add(new Person(s[0], s[1], toInt(s[2]), s[3], s[4], toInt(s[5]), toInt(s[6]), s[7]));
                            }
                        }
                    }
                }
                catch (Exception)
                {
                    MessageBox.Show("Soubor nebyl nalezen.");
                }
            }
            return people;
        }

        /// <summary>
        /// Metoda načte uložené položky (Item)
        /// <paramref name="fileType">string type</paramref>
        /// <param name="fileType">Typ souboru, který má být načten (txt, cvs, xml)</param>
        /// <paramref name="system">bool system</paramref>
        /// <param name="system">TRUE jedná-li se o načítání uložených dat při spuštění programu, FALSE otevře dialog</param>
        /// </summary>
        public BindingList<Item> loadItemsFromFile(string fileType = "txt", bool system = true)
        {
            BindingList<Item> items = new BindingList<Item>();

            OpenFileDialog openFile = new OpenFileDialog();

            /* Nastavení typu souborů - zatím je možné importovat pouze programem vytvořené TXT soubory*/
            if (fileType.CompareTo("txt") == 0)
                openFile.Filter = "Text Files (*.txt)|*.txt";
            else if (fileType.CompareTo("cvs") == 0)
                openFile.Filter = "csv files (*.csv)|*.csv";
            else if (fileType.CompareTo("xml") == 0)
                openFile.Filter = "xml files (*.xml)|*.xml";
            else
                openFile.Filter = "All Files (*.*)|*.*";

            System.Windows.Forms.DialogResult res;
            string filename;
            if (system)
            {
                res = DialogResult.OK;
                filename = "items.txt";
            }
            else
            {
                res = openFile.ShowDialog();
                filename = openFile.FileName;
            }

            if (res == System.Windows.Forms.DialogResult.OK)
            {
                try
                {
                    using (Stream file = new FileStream(filename, FileMode.Open, FileAccess.Read))
                    {
                        using (StreamReader reader = new StreamReader(file, Encoding.UTF8))
                        {
                            string line = "";
                            while ((line = reader.ReadLine()) != null)
                            {
                                String[] s = line.Split('|');
                                items.Add(new Item(toInt(s[0]), s[1], toInt(s[2]), toInt(s[3])));
                            }
                        }
                    }
                }
                catch (Exception)
                {
                    MessageBox.Show("Soubor nebyl nalezen.");
                }
            }
            return items;
        }

        /// <summary>
        /// Metoda načte uložené faktury (Invoice)
        /// <paramref name="fileType">string type</paramref>
        /// <param name="fileType">Typ souboru, který má být načten (txt, cvs, xml) - výchozí hodnota je "txt"</param>
        /// <paramref name="system">bool system</paramref>
        /// <param name="system">TRUE (výchozí hodnota) jedná-li se o načítání uložených dat při spuštění programu, FALSE otevře dialog</param>
        /// </summary>
        public BindingList<Invoice> loadInvoicesFromFile(string fileType = "txt", bool system = true)
        {
            BindingList<Invoice> invoices = new BindingList<Invoice>();

            OpenFileDialog openFile = new OpenFileDialog();

            /* Nastavení typu souborů - zatím je možné importovat pouze programem vytvořené TXT soubory*/
            if (fileType.CompareTo("txt") == 0)
                openFile.Filter = "Text Files (*.txt)|*.txt";
            else if (fileType.CompareTo("cvs") == 0)
                openFile.Filter = "csv files (*.csv)|*.csv";
            else if (fileType.CompareTo("xml") == 0)
                openFile.Filter = "xml files (*.xml)|*.xml";
            else
                openFile.Filter = "All Files (*.*)|*.*";

            System.Windows.Forms.DialogResult res;
            string filename;
            if (system)
            {
                res = DialogResult.OK;
                filename = "invoices.txt";
            }
            else
            {
                res = openFile.ShowDialog();
                filename = openFile.FileName;
            }

            if (res == System.Windows.Forms.DialogResult.OK)
            {
                try
                {
                    using (Stream file = new FileStream(filename, FileMode.Open, FileAccess.Read))
                    {
                        using (StreamReader reader = new StreamReader(file, Encoding.UTF8))
                        {
                            string line = "";
                            while ((line = reader.ReadLine()) != null)
                            {
                                String[] s = line.Split('|');
                                Person subscriber = findPersonById(toInt(s[2]), Core.people);
                                Person supplier = findPersonById(toInt(s[3]), Core.people);

                                //převede datum ze stringu do formátu DateTime
                                DateTimeFormatInfo formatInfo = new DateTimeFormatInfo();
                                formatInfo.ShortDatePattern = "dd-MM-yyyy";
                                formatInfo.DateSeparator = ".";
                                DateTime date = Convert.ToDateTime(s[6], formatInfo);

                                invoices.Add(new Invoice(toInt(s[0]), s[1], subscriber, supplier, toInt(s[4]), toBoolean(s[5]), date));
                            }
                        }
                    }
                }
                catch (Exception)
                {
                    MessageBox.Show("Soubor nebyl nalezen.");
                }
            }
            return invoices;
        }


        /// <summary>
        /// Metoda převeda řetězec (String) na číslo (Int32)
        /// <paramref name="s">String s</paramref>
        /// <param name="s">číslo uložené jako řetězec</param>
        /// </summary>
        public static int toInt(String s)
        {
            int i = int.MinValue;
            try
            {
                i = Convert.ToInt32(s);
            }
            catch (FormatException e)
            {
                MessageBox.Show("Error: Input string is not a sequence of digits."+e.Message);
            }
            catch (OverflowException e)
            {
                MessageBox.Show("Error: The number cannot fit in an Int32." + e.Message);
            }
            return i;
        }

        /// <summary>
        /// Metoda převeda řetězec (String) na logickou hodnotu (Boolean)
        /// <paramref name="value">String value</paramref>
        /// <param name="value">logická hodnota uložená jako řetězec</param>
        /// </summary>
        private static bool toBoolean(string value)
        {
            bool b = false;
            try
            {
                b = Convert.ToBoolean(value);
            }
            catch (FormatException)
            {
                Console.WriteLine("Unable to convert '{0}' to a Boolean.", value);
            }
            return b;
        }

        /// <summary>
        /// Metoda najde osobu (Person) podle jejího IČ/ID
        /// <paramref name="id">int id</paramref>
        /// <paramref name="people">BindingList<Person> people</paramref>
        /// <param name="id">IČ/ID osoby</param>
        /// <param name="people">seznam všech osob</param>
        /// </summary>
        public static Person findPersonById(int id, BindingList<Person> people)
        {
            foreach (Person os in people)
            {
                if (os.CompanyId == id)
                    return os;
            }
            return null;
        }
    }
}
