﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using Data;

namespace App
{
    /// <summary>
    /// Formulář pro úpravu faktury (Invoice)
    /// </summary>
    public partial class InvoiceEdit : Form
    {
        private ListBox list;
        private Boolean error = false;
        private List<String> errMsg = new List<string>();
        private Invoice inv;
        
        /// <summary>
        /// Načte data vybrané faktury do formulářových prvků
        /// </summary>
        /// <param name="list">ListBox</param>
        public InvoiceEdit(ListBox list)
        {
            this.list = list;
            InitializeComponent();

            invoiceCtrl1.TextBoxId.ReadOnly = true;
            this.inv = list.SelectedItem as Invoice;

            invoiceCtrl1.TextBoxType.Text = inv.Name;
            invoiceCtrl1.TextBoxId.Text = inv.ID.ToString();

            /*Vypíše všechny osoby a vybere tu, která je přidáná na faktruře*/
            foreach (Person p in Core.people)
                invoiceCtrl1.ComboSupplier.Items.Add(p.Name + " " + p.Surname);
            invoiceCtrl1.ComboSupplier.SelectedIndex = invoiceCtrl1.ComboSupplier.FindString(inv.Supplier.Name + " " + inv.Supplier.Surname);
            foreach (Person p in Core.people)
                invoiceCtrl1.ComboSubscriber.Items.Add(p.Name + " " + p.Surname);
            invoiceCtrl1.ComboSubscriber.SelectedIndex = invoiceCtrl1.ComboSubscriber.FindString(inv.Subscriber.Name + " " + inv.Subscriber.Surname);

            invoiceCtrl1.DateTimeCreated.Text = inv.Created.ToString();
            invoiceCtrl1.DateTimeDueDate.Text = inv.DueDate.ToString();
            invoiceCtrl1.TextBoxDueDays.Text = inv.DueDays.ToString();

            if (inv.Paid)
                invoiceCtrl1.CheckBoxPaid.Checked = true;

            foreach (Item it in Core.items)
            {
                invoiceCtrl1.CheckedListBoxInv.Items.Add(it);
            }
        }

        /// <summary>
        /// Uzavře formulář bez uložení dat, klávesová zkratka ESC.
        /// </summary>
        /// <param name="sender">object</param>
        /// <param name="e">EventArgs</param>
        private void buttonCancel_Click(object sender, EventArgs e)
        {
            Close();
        }

        /// <summary>
        /// Uloží pozměněné informace o faktruře.
        /// </summary>
        /// <param name="sender">object<param>
        /// <param name="e">EventArgs</param>
        private void buttonOk_Click(object sender, EventArgs e)
        {
            if (invoiceCtrl1.TextBoxType.Text != "" && (
                invoiceCtrl1.TextBoxType.Text.CompareTo("Přijatá") == 0 ||
                invoiceCtrl1.TextBoxType.Text.CompareTo("Vydaná") == 0))
            {
                inv.Name = invoiceCtrl1.TextBoxType.Text;
            }
            else
            {
                error = true;
                errMsg.Add("Typ musí být vyplněn a může nabívat pouze hodnot:\n\"Přijatá\" nebo \"Vydaná\"");
            }


            if (invoiceCtrl1.ComboSupplier.Text != "")
            {
                foreach (Person p in Core.people)
                {
                    if (invoiceCtrl1.ComboSupplier.Text.CompareTo(p.Name + " " + p.Surname) == 0)
                    {
                        inv.Supplier = p;
                    }
                }
            }
            else
            {
                error = true;
                errMsg.Add("Přijmení nemůže být prázdné");
            }

            if (invoiceCtrl1.ComboSubscriber.Text != "")
            {
                foreach (Person p in Core.people)
                {
                    if (invoiceCtrl1.ComboSubscriber.Text.CompareTo(p.Name + " " + p.Surname) == 0)
                    {
                        inv.Subscriber = p;
                    }
                }
            }
            else
            {
                error = true;
                errMsg.Add("Přijmení nemůže být prázdné");
            }

            inv.Paid = invoiceCtrl1.CheckBoxPaid.Checked;
            inv.Created = DateTime.Parse(invoiceCtrl1.DateTimeCreated.Text);
            if (invoiceCtrl1.TextBoxDueDays.Text != "")
            {
                inv.DueDays = int.Parse(invoiceCtrl1.TextBoxDueDays.Text);
            }
            else
            {
                error = true;
                errMsg.Add("Počet dní do splatnosti nemůže být prázdný");
            }

            foreach (Item item in invoiceCtrl1.CheckedListBoxInv.CheckedItems)
            {
                inv.addItem(item);
            }


            if (error)
            {
                string str = "";
                foreach (String s in errMsg)
                {
                    str += s + "\n";
                    errMsg.Remove(s);
                }
                MessageBox.Show(str);
                error = false;
            }
            else
            {
                Close();
            }
        }

        /// <summary>
        /// Klávesové zkratky pro práci s formulářem.
        /// ENTER = uložit
        /// ESC = zavřít
        /// </summary>
        /// <param name="sender">object</param>
        /// <param name="e">KeyEventArgs</param>
        public void KeyUpEvent(object sender, KeyEventArgs e) //aby to fungovalo musí se u okna zapnout KeyPreview!!!
        {
            switch (e.KeyCode)
            {
                case Keys.Enter:
                    buttonOk.PerformClick();
                    break;

                case Keys.Escape:
                    buttonCancel.PerformClick();
                    break;

                default:
                    break;
            }
        }

        /// <summary>
        /// Metoda zkontroluje jestli nové ID nemá již nějaká jiná faktura.
        /// <param name="id">Integer</param>
        /// </summary>
        private bool checkId(int id)
        {
            foreach (Person person in Core.people)
            {
                if (person.CompanyId == id)
                {
                    MessageBox.Show("Faktura s tímto ID již existuje.");
                    return false;
                }
            }
            return true;
        }
    }
}
