﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using Data;

namespace App
{
    /// <summary>
    /// Formulář pro úpravu položky (Item)
    /// </summary>
    public partial class ItemEdit : Form
    {
        private ListBox list;
        private Boolean error = false;
        private List<String> errMsg = new List<string>();
        private Item item;

        /// <summary>
        /// Načte data vybrané položky do formulářových prvků
        /// </summary>
        /// <param name="list">ListBox</param>
        public ItemEdit(ListBox list)
        {
            this.list = list;
            InitializeComponent();

            itemCtrl1.TextBoxId.ReadOnly = true;
            this.item = list.SelectedItem as Item;

            itemCtrl1.TextBoxId.Text = item.ID.ToString();
            itemCtrl1.TextBoxDesc.Text = item.Description;
            itemCtrl1.TextBoxQuantity.Text = item.Quantity.ToString();
            itemCtrl1.TextBoxPrice.Text = item.PricePerQ.ToString();
        }

        /// <summary>
        /// Uzavře formulář bez uložení dat, klávesová zkratka ESC.
        /// </summary>
        /// <param name="sender">object</param>
        /// <param name="e">EventArgs</param>
        private void buttonCancel_Click(object sender, EventArgs e)
        {
            Close();
        }

        /// <summary>
        /// Klávesové zkratky pro práci s formulářem.
        /// ENTER = uložit
        /// ESC = zavřít
        /// </summary>
        /// <param name="sender">object</param>
        /// <param name="e">KeyEventArgs</param>
        public void KeyUpEvent(object sender, KeyEventArgs e) //aby to fungovalo musí se u okna zapnout KeyPreview!!!
        {
            switch (e.KeyCode)
            {
                case Keys.Enter:
                    buttonOk.PerformClick();
                    break;

                case Keys.Escape:
                    buttonCancel.PerformClick();
                    break;

                default:
                    break;
            }
        }

        /// <summary>
        /// Uloží pozměněné informace o osobě
        /// </summary>
        /// <param name="sender"object></param>
        /// <param name="e">EventArgs</param>
        private void buttonOk_Click(object sender, EventArgs e)
        {
            if (itemCtrl1.TextBoxDesc.Text != "")
            {
                item.Description = itemCtrl1.TextBoxDesc.Text;
            }
            else
            {
                error = true;
                errMsg.Add("Popis nemůže být prázdný");
            }

            if (itemCtrl1.TextBoxQuantity.Text != "")
            {
                item.Quantity = int.Parse(itemCtrl1.TextBoxQuantity.Text);
            }
            else
            {
                error = true;
                errMsg.Add("Množství nemůže být prázdné");
            }

            if (itemCtrl1.TextBoxPrice.Text != "")
            {
                item.PricePerQ = int.Parse(itemCtrl1.TextBoxPrice.Text);
            }
            else
            {
                error = true;
                errMsg.Add("Cena za jednotku nemůže být prázdná");
            }

            if (error)
            {
                string str = "";
                foreach (String s in errMsg)
                {
                    str += s + "\n";
                    errMsg.Remove(s);
                }
                MessageBox.Show(str);
                error = false;
            }
            else
            {
                Core.items.Add(item);
                Close();
            }
        }
    }
}
