﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using System.Windows.Forms;
using Data;
using System.Xml;

namespace App
{
    /// <summary>
    /// Třída pro exportování dat do formátu XML
    /// </summary>
    class XmlExport
    {
        /// <summary>
        /// Třída pro export dat do formátu XML
        /// </summary>
        /// <param name="what">string, parametr podle kterého se určuje jaká data se budou exportovat (people, invoices, items, all)</param>
        public XmlExport(string what = "all")
        {
            // Displays a SaveFileDialog so the user can save the Image
            // assigned to Button2.
            SaveFileDialog saveFileDialog1 = new SaveFileDialog();
            saveFileDialog1.Filter = "xml files (*.xml)|*.xml";
            saveFileDialog1.Title = "Export do XML";
            saveFileDialog1.ShowDialog();

            // If the file name is not an empty string open it for saving.
            if (saveFileDialog1.FileName != "")
            {
                XmlWriterSettings setting = new XmlWriterSettings();
                setting.Indent = true;
                XmlWriter writer = XmlWriter.Create(saveFileDialog1.FileName, setting);
                writer.WriteStartDocument();

                switch (what.ToLower())
                {
                    case "people":
                        writePeople(writer);
                        break;

                    case "invoices":
                        writeInvoices(writer);
                        break;

                    case "items":
                        writeItems(writer);
                        break;

                    default:
                        writeAll(writer);
                        break;
                }
                    
                writer.WriteEndDocument();
                writer.Close();

                MessageBox.Show("XML vytvořeno :-)");
            }
        }

        /// <summary>
        /// Zápis všech dat (osob, faktru a položek) do formátu XML
        /// </summary>
        /// <param name="writer">XmlWriter</param>
        public void writeAll(XmlWriter writer)
        {
            writer.WriteStartElement("InvoiceManager");
            writePeople(writer);
            writeInvoices(writer);
            writeItems(writer);
            writer.WriteEndElement();
        }

        /// <summary>
        /// Zápis osob do formátu XML
        /// </summary>
        /// <param name="writer">XmlWriter</param>
        public void writePeople(XmlWriter writer)
        {
            writer.WriteStartElement("people");
            foreach (Person p in Core.people)
            {
                writer.WriteStartElement("person");
                writer.WriteElementString("id", p.CompanyId.ToString());
                writer.WriteElementString("name", p.Name);
                writer.WriteElementString("surname", p.Surname);
                writer.WriteElementString("bankAccount", p.BankAccount);
                writer.WriteElementString("taxId", p.TaxId);
                writer.WriteElementString("street", p.Street);
                writer.WriteElementString("nr", p.NR.ToString());
                writer.WriteElementString("postCode", p.PostCode.ToString());
                writer.WriteEndElement();
            }
            writer.WriteEndElement();
        }

        /// <summary>
        /// Zápis faktur do formátu XML
        /// </summary>
        /// <param name="writer">XmlWriter</param>
        public void writeInvoices(XmlWriter writer)
        {
            writer.WriteStartElement("invoices");
            foreach (Invoice inv in Core.invoices)
            {
                writer.WriteStartElement("invoice");
                writer.WriteElementString("id", inv.ID.ToString());
                writer.WriteElementString("name", inv.Name);
                writer.WriteElementString("subscriber", inv.Subscriber.CompanyId.ToString());
                writer.WriteElementString("supplier", inv.Supplier.CompanyId.ToString());
                writer.WriteStartElement("items");
                foreach (Item it in inv.Items)
                {
                    writer.WriteElementString("item", it.ID.ToString());
                }
                writer.WriteEndElement();
                writer.WriteElementString("created", inv.Created.ToString());
                writer.WriteElementString("dueDays", inv.DueDays.ToString());
                writer.WriteElementString("paid", inv.Paid.ToString());
                writer.WriteEndElement();
            }
            writer.WriteEndElement();
        }

        /// <summary>
        /// Zápis položek do formátu XML
        /// </summary>
        /// <param name="writer">XmlWriter</param>
        public void writeItems(XmlWriter writer)
        {
            writer.WriteStartElement("items");
            foreach (Item it in Core.items)
            {
                writer.WriteStartElement("item");
                writer.WriteElementString("id", it.ID.ToString());
                writer.WriteElementString("description", it.Description);
                writer.WriteElementString("quantity", it.Quantity.ToString());
                writer.WriteElementString("pricePerQuantity", it.PricePerQ.ToString());
                writer.WriteElementString("totalPrice", it.Price.ToString());
                writer.WriteEndElement();
            }
            writer.WriteEndElement();
        }
    }
}
